% This program modifies the feature region of a speech sound by using STFT 
% and its inverse transform. 
% Input: s - original speech
% Output: s1 - the modified speech
function s1 = modify(s, Fs, regs)

load('hear_thresh.mat');
        
% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number
delay = 736; % delay of the auditory filter introduced by AIgram 4.0
s = [zeros(delay,1); s(1:end-delay)];

%delay = 0.046; % delay of the auditory filter introduced by AIgram 4.0
%s = [zeros(delay*Fs,1);s(1:end-delay*Fs)];

framesize = 0.010; % 5ms
wsize = round(framesize*Fs);
[S, s_length] = STFT(s, wsize); 
[nframe_stft, nband_stft] = size(S); 

mask_final = ones(size(S));

% first transform the REGs in AIgrams to REGs in STFT space
for i=1:length(regs)
    
    mask = regs{i};
    t_src_AI = mask.pos(1,:);
    f_src_AI = mask.pos(2,:);
    gain = mask.gain;
    
    [t_src_STFT, f_src_STFT] = ai2stft(t_src_AI, f_src_AI, nband_stft, nframe_stft, Fs);
    
    reg_src = zeros(2, length(t_src_STFT));
    reg_src(1,:) = t_src_STFT;
    reg_src(2,:) = f_src_STFT;
    
    mask_src = gen_mask(reg_src, gain, nframe_stft, nband_stft);
    
    [nRow, nCol] = size(mask.pos);
    
    if nRow>2 % move the region to [pos(3,1) pos(3,2)]

        % remove the selected feature at the source region        
        mask_src_remove = gen_mask(reg_src, 0, nframe_stft, nband_stft);
        S1 = S.*mask_src_remove;

        % calculate the position of the destination mask on STFT plane
        t_shift_AI = mask.pos(3,1) - (max(t_src_AI) + min(t_src_AI))/2;
        f_shift_AI = mask.pos(3,2) - (max(f_src_AI) + min(f_src_AI))/2;        
        
        t_dest_AI = t_src_AI + t_shift_AI;
        f_dest_AI = f_src_AI + f_shift_AI;        
        
        [t_dest_STFT, f_dest_STFT] = ai2stft(t_dest_AI, f_dest_AI, nband_stft, nframe_stft, Fs);
        
        reg_dest = zeros(2, length(t_dest_STFT));
        reg_dest(1,:) = t_dest_STFT;
        reg_dest(2,:) = f_dest_STFT;

        % copy the selected feature from source to destination
        mask_dest = gen_mask(reg_dest, 100, nframe_stft, nband_stft);
        [t_reg_dest, f_reg_dest] = find(mask_dest==100);
        t_reg_dest = max(1, t_reg_dest); t_reg_dest = min(nframe_stft, t_reg_dest); 
        f_reg_dest = max(1, f_reg_dest); f_reg_dest = min(nband_stft, f_reg_dest); 
        
        % map the points in dest region back to the source region
        [t_reg_dest_AI, f_reg_dest_AI] = stft2ai(t_reg_dest, f_reg_dest, nband_stft, nframe_stft, Fs);
        t_reg_src_AI = t_reg_dest_AI - t_shift_AI;
        f_reg_src_AI = f_reg_dest_AI - f_shift_AI;
        [t_reg_src_STFT, f_reg_src_STFT] = ai2stft(t_reg_src_AI, f_reg_src_AI, nband_stft, nframe_stft, Fs);
        
%         num_pixel_scr_stft = length(find(mask_src_remove==0));
%         num_pixel_dest_stft = length(find(mask_dest==100));        
%         if num_pixel_dest_stft>0
%             K = num_pixel_scr_stft/num_pixel_dest_stft; % to keep energy unchanged
%         else
%             K = 1;
%         end
        
        S2 = S*0;

        f_scr = round(f_reg_src_STFT*(Fs/2/nband_stft));
        f_dest = round(f_reg_dest*(Fs/2/nband_stft));
        
        %K1 = 10.^((hear_thresh(f_dest)-hear_thresh(f_scr))/20); % equal-loudness
        
        K2 = f_reg_src_STFT./f_reg_dest; % proportional to ERB width
        
%         X = repmat([1:nframe_stft], nband_stft, 1);
%         Y = repmat([1:nband_stft]', 1, nframe_stft);
%         Z = S';
%         XI = t_reg_src_STFT; YI = f_reg_src_STFT;
%         ZI = interp2(X, Y, Z, XI, YI, 'nearest');
%             
%         for i=1:length(t_reg_dest)
%             S2(t_reg_dest(i), f_reg_dest(i)) = ZI(i)*gain*K(i); % assign the shifted feature at the dest region
%         end
%         
%         %%%%----------Debug----------%%%%
%         figure
%         subplot(121), plot3(X, Y, abs(Z)); xlim([min(XI), max(XI)]); ylim([min(YI), max(YI)]);
%         subplot(122), plot3(XI, YI, abs(ZI)); xlim([min(XI), max(XI)]); ylim([min(YI), max(YI)]);
%         %%%%----------Debug----------%%%%        
        
        for i=1:length(t_reg_dest)
            S2(t_reg_dest(i), f_reg_dest(i)) = (S(round(t_reg_src_STFT(i)), round(f_reg_src_STFT(i))))*gain*K2(i); % assign the shifted feature at the dest region
        end
        
        mask_dest = gen_mask(reg_dest, 0, nframe_stft, nband_stft);
        S1 = S1.*mask_dest;
        
        S = S1 + S2;
        
    else 
        mask_final = mask_final.*mask_src;
    end
    
end

% set the frequency component above 7.4kHz to zeros
[nframe, nfreqbin] = size(mask_final);
p = floor(7400/(Fs/2)*nfreqbin);
mask_final(:,p:end) = 0;

S = S.*mask_final;
s1 = ISTFT(S, wsize, s_length); 
%s1 = [s1(delay*Fs+1:end); zeros(delay*Fs,1)]; % shift back
s1 = [s1(delay+1:end); zeros(delay,1)]; % shift back

% figure(); 
% subplot(131), imagesc(abs(S')); title('original');
% subplot(132), imagesc(mask_final'); title('mask'); [S1, s_length] = STFT(s1, wsize); 
% subplot(133), imagesc(abs(S1')); title('modified');