% This program modifies the feature region of a speech sound by using STFT 
% and its inverse transform. 
% Input: s - original speech
% Output: s1 - the modified speech
function s1 = modify(s, Fs, mask_final)

% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number
delay = 736; % delay of the auditory filter introduced by AIgram 4.0
s = [zeros(delay,1); s(1:end-delay)];

%delay = 0.046; % delay of the auditory filter introduced by AIgram 4.0
%s = [zeros(delay*Fs,1);s(1:end-delay*Fs)];

framesize = 0.010; % 5ms
wsize = round(framesize*Fs);
[S, s_length] = STFT(s, wsize); 

[nframe_stft, nband_stft] = size(S); 

reg_final = ones(size(S));
% first transform the REGs in AIgrams to REGs in STFT space
for i=1:length(mask_final)
    
    mask = mask_final{i};
    t_AI = mask.pos(1,:);
    f_AI = mask.pos(2,:);
    gain = mask.gain;
    
    [t_STFT, f_STFT] = ai2stft(t_AI, f_AI, nband_stft, nframe_stft, Fs);
    
    reg_xy = zeros(2, length(t_STFT));
    
    reg_xy(1,:) = t_STFT;
    reg_xy(2,:) = f_STFT;
    
    reg = gen_mask(reg_xy, gain, nframe_stft, nband_stft);
    
    reg_final = reg_final.*reg';
end

% set the frequency component above 7.4kHz to zeros
[nframe, nfreqbin] = size(reg_final);
p = floor(7400/(Fs/2)*nfreqbin);
reg_final(:,p:end) = 0;

S = S.*reg_final;
s1 = ISTFT(S, wsize, s_length); 
%s1 = [s1(delay*Fs+1:end); zeros(delay*Fs,1)]; % shift back
s1 = [s1(delay+1:end); zeros(delay,1)]; % shift back

% figure(); 
% subplot(131), imagesc(abs(S')); title('original');
% subplot(132), imagesc(reg_final'); title('mask'); [S1, s_length] = STFT(s1, wsize); 
% subplot(133), imagesc(abs(S1')); title('modified');