%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is created to pull out the MN16 data of the Normal 
% Hearing human speech perception experiment, NH-HI-06.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2006-11-07
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function pull_MN16R_data(zipfile)

datapath = './logfiles/';
mkdir(datapath); 
unzip(zipfile, datapath);

files = dir([datapath '*.log']);
for i=1:length(files)
	filename = [datapath files(i).name]
    fid = fopen(filename);

    EOF = 0;
    while ~EOF
        str=[];
    	str=fgetl(fid);
    	
        if ~ischar(str), EOF=1; break, end; % Test for the end of file
    
        if strfind(str, 'LNAME')
            Listener = str(9:end); 
            if isempty(Listener), 
                Listener = ['nobody' num2str(i)]; 
            end
            str=[]; str=fgetl(fid); L1 = str(9:end);
            str=[]; str=fgetl(fid); L2 = str(9:end);
            str=[]; str=fgetl(fid); Sex = str(9:end);
            str=[]; str=fgetl(fid); Age = str(7:end);          
        end

        if strfind(str, 'Begin Data')
            str=[]; str=fgetl(fid);
            Data = textscan(fid, 'FILE=s_%s STIM=%s RESP=%s SNR=%d TIME=%f PLAYS=%d %*[^\n]');
            
            [nRow, nCol] = size(Data{1});
            for j = 1:nRow
                Data{1}{j} = Data{1}{j}(1:end-4);
                DataListener{j} = Listener;
                DataL1{j} = L1;
                DataL2{j} = L2;
                DataSex{j} = Sex;
                DataAge{j} = Age;
            end
        end

    end
    fclose(fid);
    
    [nRow, nCol] = size(Data);
    Data{nCol+1} = DataListener'; DataListener = '';
    Data{nCol+2} = DataL1'; DataL1 = '';
    Data{nCol+3} = DataL2'; DataL2 = '';
    Data{nCol+4} = DataSex'; DataSex = '';
    Data{nCol+5} = DataAge'; DataAge = '';
    
    if isequal(i, 1) 
        NHDB = Data;
    else
        [nRow nCol] = size(Data);
        for k=1:nCol
            NHDB{k} = cat(1, NHDB{k}, Data{k});
        end
    end

end

% Pull out basic information from the Normal Hearing data NHDB
Talker_Stim = unique(sort(NHDB{1}))
Listener = unique(sort(NHDB{7}))
Stim = unique(sort(NHDB{2}))
Resp = unique(sort(NHDB{3}))
SNR = unique(sort(NHDB{4}))

save('MN16R.mat', 'NHDB', 'Talker_Stim', 'Listener', 'Stim', 'Resp', 'SNR');

rmdir(datapath,'s');


